#include "C:\\PIC\\dro452\\dro_Auto.h"

#include <Delays.h>
#include <datalib.h>
#include <strings.h>

#pragma __config _CONFIG1H,_HSPLL_OSC_1H
#pragma __config _CONFIG2L,_PWRT_ON_2L & _BOR_ON_2L
#pragma __config _CONFIG2H,_WDT_OFF_2H
#pragma __config _CONFIG4L,_LVP_OFF_4L & _STVR_ON_4L

#define LOW		0
#define HIGH	1

#define ON		1
#define OFF		0

#define SKIP    1
#define NO_SKIP 0

#define TRUE	1
#define FALSE	-1

#define WAIT    1
#define NO_WAIT 0

#define RIGHT	1
#define LEFT	0

#define MAG_X2	2
#define MAG_X1	1


#define SET			0x01
#define HALF		0x02
#define CANSEL		0x03
#define DOWN		0x04

#define KEY_MINUS	0x05
#define KEY_DOT		0x06
#define KEY_0		0x07
#define UP			0x08

#define KEY_3		0x09
#define KEY_2		0x0a
#define KEY_1		0x0b
#define AXIS_Z		0x0c

#define KEY_6		0x0d
#define KEY_5		0x0e
#define KEY_4		0x0f
#define AXIS_Y		0x10

#define KEY_9		0x11
#define KEY_8		0x12
#define KEY_7		0x13
#define AXIS_X		0x14

#define KEY_NON     0xFF

// transmits previously from a low bit
#define LED_Dig1_MINUS  0x0080
#define LED_Dig1_0      0x0707
#define LED_Dig1_1      0x0404
#define LED_Dig1_2      0x0683
#define LED_Dig1_3      0x0686
#define LED_Dig1_4      0x0584
#define LED_Dig1_5      0x0386
#define LED_Dig1_6      0x0387
#define LED_Dig1_7      0x0604
#define LED_Dig1_8      0x0787
#define LED_Dig1_9      0x0786
#define LED_Dig1_DOT    0x0000
#define LED_Dig2_MINUS  0x0040
#define LED_Dig2_0      0x7830
#define LED_Dig2_1      0x6000
#define LED_Dig2_2      0x5070
#define LED_Dig2_3      0x7060
#define LED_Dig2_4      0x6840
#define LED_Dig2_5      0x3860
#define LED_Dig2_6      0x3870
#define LED_Dig2_7      0x7000
#define LED_Dig2_8      0x7870
#define LED_Dig2_9      0x7860
#define LED_Dig2_DOT    0x8000

#define LED_ON     0x0008

#define NO_SIGN		0x01
#define NO_SPAC		0x02
#define NO_ZERO     0x04

/*---- EEPROM address ---------------------------*/
#define EE_ADDR_X	0x00
#define EE_ADDR_Y	0x10
#define EE_ADDR_Z   0x20

/*---- calibration         offset address -------*/
#define CAL_OFFSET	0
/*---- increment direction offset address -------*/
#define DIR_OFFSET  4
/*---- magnification       offset address -------*/
#define MAG_OFFSET  5
/*---- add axis            offset address -------*/
#define ADD_OFFSET  6
/*---- formula             offset address -------*/
#define FML_OFFSET  7

#define bit_clear(reg,bit)		(reg&=~(1<<bit))
#define bit_set(reg,bit)		(reg|=(1<<bit))
#define bit_is_set(reg,bit) 	((reg & (1<<bit)) == (1<<bit))
#define bit_is_clear(reg,bit)	((reg & (1<<bit)) == 0x00)

#define abs(val)  ( (val)>=0 ? (val) : -(val) )
#define max(a,b)  ( (a)>(b) ? (a) : (b) )
#define min(a,b)  ( (a)<(b) ? (a) : (b) )

typedef union {
	char c_dat[2];
	int  i_dat;
} INT_CHAR;

typedef union {
	char c_dat[4];
	long l_dat;
} LONG_CHAR;

void lcd_init();
void lcd_clear();
void lcd_locate(unsigned char,unsigned char);
void lcd_char(unsigned char);
void lcd_string(unsigned char *);

void exec_command();
unsigned char key_input();
unsigned char key_down( unsigned char);
unsigned char key_char(unsigned char);
void key_up();
void key_up_anykey();
void beep(int);
long get_numeric( char *exception );
long get_long( char *exception );
void long_to_string(unsigned char *,long,unsigned char,unsigned char,unsigned char);
long str_to_long( unsigned char * );
void lng_2_str( long, unsigned char * );
void led_off();
void led_out( unsigned char * );
void axis_calibration( char );
void lcd_clear_line( unsigned char );
void cal_set();
void axis_set_value( char );
long calc_cal( long , long , long );

void i2c_init();
void i2c_startCon();
void i2c_send( unsigned char );
void i2c_stopSeq();
void i2c_restart();
unsigned char i2c_recv();

void rs232_init();
void rs232_send( unsigned char );
unsigned char rs232_recv();

long ee_long_read( unsigned char );
char ee_char_read( unsigned char );
void ee_long_write( unsigned char, long );
void ee_char_write( unsigned char, char );

void 		  ser_eep_write( unsigned char, unsigned char, unsigned char, unsigned char);
unsigned char ser_eep_read(  unsigned char, unsigned char, unsigned char );

long get_caliper_data_X();
long get_caliper_data_Y();
long get_caliper_data_Z();

long disp_to_count( long , long );

void wait_us( unsigned char );
#pragma asmfunc wait_us

// unsigned char bit_counter;
unsigned char XYZ_counter;
unsigned char Key_col;
unsigned char Key_row;
unsigned char cansel_flag;

long X_data_value;
long Y_data_value;
long Z_data_value;
long X_data_value_sav;
long Y_data_value_sav;
long Z_data_value_sav;

long X_offset;
long Y_offset;
long Z_offset;

long X_axis_cal;
long Y_axis_cal;
long Z_axis_cal;

char X_axis_dir;
char Y_axis_dir;
char Z_axis_dir;

char X_axis_mag;
char Y_axis_mag;
char Z_axis_mag;

char x_str[12];
char y_str[12];
char z_str[12];

//
// This file includes all user definable routines. It may be changed at will as
// it will not be regenerated once the application has been generated for the
// first time.
//

//*******************************************************************************
//
// Insert your interrupt handling code if required here. 
// Note quick interrupts are used so code must be simple
// See the manual for details of quick interrupts.
//

// const int NormalInt=1;

void UserInterrupt()
{
 #asmline goto UserIntReturn	; PIC Assembler - go back to interrupt routine
}


//*******************************************************************************
//
// Insert your initialisation code if required here. 
// Note that when this routine is called Interrupts will not be enabled - the 
// Application Designer will enable them before the main loop
//

void UserInitialise()
{
//	char buf[10];

	led_off();
	
    /*--- PORTA DIGITAL Input/Output ----------*/
	ADCON1=0x06;
	/*--- PORTB pullup disable ---*/
	bit_set( INTCON2, bNOT_RBPU );

	/*--- Disable proority level inturrupt ----*/
	bit_clear( RCON, IPEN  );

	/*--- Disable interrupts ----*/
	bit_clear( INTCON, GIE  );

	lcd_init();
	Wait(320);

	lcd_string("Initialize...");
	beep(500);
				
//	rs232_init();
//	rs232_send('A');
	
	XYZ_counter = 'Z';

	Key_row = 1;
	Key_col = 1;

	X_offset = 0L;
	Y_offset = 0L;
    Z_offset = 0L;

	X_data_value_sav = 0L;
	Y_data_value_sav = 0L;
	Z_data_value_sav = 0L;

	/*--- timer0 setup --*/
	bit_set(   T0CON, TMR0ON );		// use timer0
	bit_clear( T0CON, T08BIT );		// 16 bit mode
	bit_clear( T0CON, T0CS   );		// use internal clock
	bit_clear( T0CON, PSA    );		// use prescaler
	bit_set( T0CON, T0PS2  );		// prescaler 1:256
	bit_set( T0CON, T0PS1  );
	bit_set( T0CON, T0PS0  );

	cal_set();			// calibration data from EEPROM
	
	lcd_clear();
	if( X_axis_cal == -1L ) { X_axis_cal = 125; }
	if( X_axis_cal == -1L ) { Y_axis_cal = 125; }
	if( X_axis_cal == -1L ) { Z_axis_cal = 125; }
}

//*******************************************************************************
//
// Insert your main loop code if required here. This routine will be called
// as part of the main loop code
//

void UserLoop()
{
	long  t_long;
	
	t_long = 0L;
	KEY_Col1 = LOW;
	KEY_Col2 = LOW;	
	KEY_Col3 = LOW;
	KEY_Col4 = LOW;
	/* detect push any key */
	if( KEY_Row1 == LOW ||
	    KEY_Row2 == LOW ||
	    KEY_Row3 == LOW ||
	    KEY_Row4 == LOW ||
	    KEY_Row5 == LOW ) {
	    led_off();
    	exec_command();
		key_up_anykey();
    }

	//  interval FE79 = 10ms = 100000 count at 40MHz
	//     prescaler 1:256
	TMR0H = 0xFE;
	TMR0L = 0x80;
	bit_clear( INTCON, TMR0IF );
	Wait(1);
	RBB = 1;

	if( XYZ_counter == 'X' ) {
		XYZ_counter = 'Y';
		Y_data_value = get_caliper_data_Y();
		if( Y_data_value != 88888 ) {
			lcd_locate( 0,1 );
			lcd_string( "Y=" );
			t_long = Y_data_value - Y_offset;
			if( Y_axis_cal > 0 ) {
				t_long += t_long/Y_axis_cal;
			} else {
				t_long -= t_long/(-Y_axis_cal);
			}
			if( Y_axis_dir == LEFT   ) { t_long = -t_long; }
			if( Y_axis_mag == MAG_X2 ) { t_long *= 2;	   } 
			long_to_string( y_str, t_long, 3, 2, 0x00 );
			lcd_string( y_str );
		}
		led_out( y_str );
	}

	if( XYZ_counter == 'Y' ) {
		XYZ_counter = 'Z';
		Z_data_value = get_caliper_data_Z();
		if( Z_data_value != 88888 ) {
			lcd_locate( 0,2 );
			lcd_string( "Z=" );
			t_long = Z_data_value - Z_offset;
			if( Z_axis_cal > 0 ) {
				t_long += t_long/Z_axis_cal;
			} else {
				t_long -= t_long/(-Z_axis_cal);
			}
			if( Z_axis_dir == LEFT )   { t_long = -t_long; } 
			if( Z_axis_mag == MAG_X2 ) { t_long *= 2;	   } 			
			long_to_string( z_str, t_long, 3, 2, 0x00 );
			lcd_string( z_str );
		}
		led_out( z_str );
	}

	if( XYZ_counter == 'Z' ) {
		XYZ_counter = 'X';
		X_data_value = get_caliper_data_X();
		if( X_data_value != 88888 ) {
			lcd_locate( 0,0 );
			lcd_string( "X=" );
			t_long = X_data_value - X_offset;
			if( X_axis_cal > 0 ) {
				t_long += t_long/X_axis_cal;
			} else {
				t_long -= t_long/(-X_axis_cal);
			}
			if( X_axis_dir == LEFT )   { t_long = -t_long; } 
			if( X_axis_mag == MAG_X2 ) { t_long *= 2;	   } 
			long_to_string( x_str, t_long, 3, 2, 0x00 );
			lcd_string( x_str );
		}
   		led_out( x_str );
	}
	
	RBB = 0;
}

//
// User occurrence code
//


/*----------------------------------------------------------------------------*/
/* function: beep                                                             */
/*----------------------------------------------------------------------------*/
void beep(int time)
{
	BEEP = ON;
	Wait(time);
	BEEP = OFF;
}

/*----------------------------------------------------------------------------*/
/* function: set calibration valriable                                        */
/*----------------------------------------------------------------------------*/
void cal_set()
{
	char	buf[21];
	char	dir;
	char	magbuf[3];
	
	X_axis_cal = ee_long_read(EE_ADDR_X + CAL_OFFSET);
	Y_axis_cal = ee_long_read(EE_ADDR_Y + CAL_OFFSET);
	Z_axis_cal = ee_long_read(EE_ADDR_Z + CAL_OFFSET);

	X_axis_dir = ee_char_read(EE_ADDR_X + DIR_OFFSET);
	Y_axis_dir = ee_char_read(EE_ADDR_Y + DIR_OFFSET);
	Z_axis_dir = ee_char_read(EE_ADDR_Z + DIR_OFFSET);

	X_axis_mag = ee_char_read(EE_ADDR_X + MAG_OFFSET);
	Y_axis_mag = ee_char_read(EE_ADDR_Y + MAG_OFFSET);
	Z_axis_mag = ee_char_read(EE_ADDR_Z + MAG_OFFSET);
	
	lcd_clear();
	lcd_locate(0,0);
	if( X_axis_dir == LEFT ) dir = 'L'; else dir = 'R';
	if( X_axis_mag == MAG_X2 ) sprintf( magbuf, "x2" );	else sprintf( magbuf, "x1" );
	sprintf(buf, "X CL=%4ld DR=%c MG=%s", X_axis_cal, dir, magbuf );
	lcd_string( buf );

	lcd_locate(0,1);
	if( Y_axis_dir == LEFT ) dir = 'L'; else dir = 'R';
	if( Y_axis_mag == MAG_X2 ) sprintf( magbuf, "x2" );	else sprintf( magbuf, "x1" );
	sprintf(buf, "Y CL=%4ld DR=%c MG=%s", Y_axis_cal, dir, magbuf );	
	lcd_string( buf );

	lcd_locate(0,2);
	if( Z_axis_dir == LEFT ) dir = 'L'; else dir = 'R';
	if( Z_axis_mag == MAG_X2 ) sprintf( magbuf, "x2" );	else sprintf( magbuf, "x1" );
	sprintf(buf, "Z CL=%4ld DR=%c MG=%s", Z_axis_cal, dir, magbuf );
	lcd_string( buf );
	
	Wait(3000);
	lcd_clear();				
}


/*----------------------------------------------------------------------------*/
/* function:                                                                  */
/*----------------------------------------------------------------------------*/
void rs232_init()
{
	TXSTA = 0x20;
	SPBRG = 32;		// BAUD 9600 bps
	RCSTA = 0x90;
}


/*----------------------------------------------------------------------------*/
/* function:                                                                  */
/*----------------------------------------------------------------------------*/
void rs232_send( unsigned char data )
{
	while( bit_is_clear( TXSTA, TRMT ) ) {
		;
	}
	TXREG = data;
}	


/*----------------------------------------------------------------------------*/
/* function:                                                                  */
/*----------------------------------------------------------------------------*/
unsigned char rs232_recv()
{
	while( bit_is_clear(PIR1,RCIF) ) {
		;
	}
	if( bit_is_set( RCSTA, FERR ) || bit_is_set( RCSTA, OERR ) ) {
		// frame error
		// over run erro
	}
	
	return RCREG;
}


/*----------------------------------------------------------------------------*/
/* function:                                                                  */
/*----------------------------------------------------------------------------*/
void ser_eep_write(
 unsigned char dev_addr,
 unsigned char addr_high,
 unsigned char addr_low,
 unsigned char data )
{
	i2c_startCon();	
	i2c_send( dev_addr  );
	i2c_send( addr_high );
	i2c_send( addr_low  );
    i2c_send( data      );
	i2c_stopSeq();
	Wait(5);  
}

/*----------------------------------------------------------------------------*/
/* function:                                                                  */
/*----------------------------------------------------------------------------*/
unsigned char ser_eep_read(
 unsigned char dev_addr,
 unsigned char addr_high,
 unsigned char addr_low )
{
	unsigned char ret_val;
	
	i2c_startCon();	
	i2c_send( dev_addr  );
	i2c_send( addr_high );
	i2c_send( addr_low  );
	i2c_restart();
    i2c_send( dev_addr | 0x01 );
	ret_val = i2c_recv();
	i2c_stopSeq();
	
	 return ret_val;  
}

/*----------------------------------------------------------------------------*/
/* function:                                                                  */
/*----------------------------------------------------------------------------*/
void i2c_init()
{
    // set I2C Master Mode 
	SSPCON1  = 0x28;
	SSPCON2 = 0x00;
//	bit_set(SSPCON2,ACKEN);		// ack sequence enable
//	bit_set(SSPCON2,RCEN);		// recieve enable
//  bit_set(SSPCON2,SEN);		// start condition enable

	SSPADD = 0x50;
}


/*----------------------------------------------------------------------------*/
/* function:                                                                  */
/*----------------------------------------------------------------------------*/
void i2c_startCon()
{
	do { 
		bit_clear(SSPCON1,WCOL); 
		bit_set(SSPCON2,SEN);
    } while( bit_is_set(SSPCON1,WCOL) )   

	while( bit_is_set(SSPCON2,SEN) ) {
		;
	}
}

/*----------------------------------------------------------------------------*/
/* function:                                                                  */
/*----------------------------------------------------------------------------*/
void i2c_restart()
{
	bit_clear(PIR1,SSPIF);
	
	bit_set(SSPCON2,RSEN);
	while( bit_is_set(SSPCON2,RSEN) ) {
		;
	}
	
	bit_clear(PIR1,SSPIF);
}

/*----------------------------------------------------------------------------*/
/* function:                                                                  */
/*----------------------------------------------------------------------------*/
void i2c_stopSeq()
{
	bit_clear(PIR1,SSPIF);
	
	bit_set(SSPCON2,PEN);
	while( bit_is_set(SSPCON2,PEN) ) {
		;
	}
	
	bit_clear(PIR1,SSPIF);
}

void i2c_send( unsigned char data )
{
	bit_clear(PIR1,SSPIF);
	SSPBUF = data;
	while( bit_is_clear(PIR1,SSPIF) ) {
		;
	}
	if( bit_is_set(SSPCON2,ACKSTAT) ) {
		;	
	} 
}


/*----------------------------------------------------------------------------*/
/* function: I2C Recieve char                                                 */
/*----------------------------------------------------------------------------*/
unsigned char i2c_recv()
{
	bit_clear(PIR1,SSPIF);

	bit_set(SSPCON2,RCEN);
	while( bit_is_set(SSPCON2,RCEN) ) {
		;
	}
	bit_set( SSPCON2, ACKDT );
	bit_set( SSPCON2, ACKEN );
	while( bit_is_set(SSPCON2,ACKEN) ) {
		;
	}
	
	return SSPBUF;
}


/*----------------------------------------------------------------------------*/
/* function: Process key INPUT                                                */
/*----------------------------------------------------------------------------*/
void exec_command()
{
	switch( key_char( SKIP ) ) {
 	  case AXIS_X:	axis_set_value( 'X' ); break;
 	  case AXIS_Y:	axis_set_value( 'Y' ); break;
 	  case AXIS_Z:	axis_set_value( 'Z' ); break;
	  break;
	}
}


/*----------------------------------------------------------------------------*/
/* function: start point set                                                  */
/*                                                                            */
/*  arg1: AXIS  'X' -> AXIS-X                                                 */
/*              'Y' -> AXIS-Y                                                 */
/*              'Z' -> AXIS-Z                                                 */
/*----------------------------------------------------------------------------*/
void axis_set_value( char axis_char )
{
	unsigned char ch;
	char		  exception;
	long		  input_num;

	lcd_clear_line( 3 );
	lcd_locate( 0, 3 );
	lcd_char( axis_char );
	lcd_char( '=' );
	exception = axis_char;
	input_num = get_numeric(&exception);
	switch( exception ) {
		case 'C':
			lcd_clear();
			break;
		case 'X':
		case 'Y':
		case 'Z':
			lcd_clear_line( 3 );
			lcd_locate( 0, 3 );
			lcd_char( axis_char );
			lcd_string( "=ZERO" );
			switch( ch = key_char(NO_SKIP) ) {
				case DOWN:
					switch( axis_char ) {
						case 'X': axis_calibration( 'X' ); break;
						case 'Y': axis_calibration( 'Y' ); break;
						case 'Z': axis_calibration( 'Z' ); break;
					}
					break;
				case SET:
					switch( axis_char ) {
						case 'X': X_offset = X_data_value; break;
						case 'Y': Y_offset = Y_data_value; break;
						case 'Z': Z_offset = Z_data_value; break;
					}
					Wait( 500 );
					lcd_clear_line( 3 );
				 	break;
				default:
					lcd_clear_line( 3 );
					lcd_locate( 0, 3 );
					lcd_string("Cansel...");
					Wait( 500 );
					lcd_clear_line( 3 );
					break;
			}
			break;
		default:
			switch( axis_char ) {
				case 'X': 
					X_offset = X_data_value - disp_to_count( input_num, X_axis_cal );
					if( X_axis_dir == LEFT )
						input_num = -input_num;
					break;	 
				case 'Y':
					Y_offset = Y_data_value - disp_to_count( input_num, Y_axis_cal );
					if( X_axis_dir == LEFT )
						input_num = -input_num;
					break;	 
				case 'Z':
					Z_offset = Z_data_value - disp_to_count( input_num, Z_axis_cal );
					if( X_axis_dir == LEFT )
						input_num = -input_num;
					break;	 
			}
			break;
	}
}


/*----------------------------------------------------------------------------*/
/* function: display -> caliper count                                         */
/*                                                                            */
/* arg1: display Value                                                        */
/* arg2: caliper count Value                                                  */
/*----------------------------------------------------------------------------*/
long disp_to_count( long disp_num, long axis_cal )
{
	return ( disp_num * axis_cal ) / ( axis_cal + 1 ); 
}


/*----------------------------------------------------------------------------*/
/* function: calibrate axis value                                             */
/*                                                                            */
/*  arg1: AXIS                                                                */
/*----------------------------------------------------------------------------*/
void axis_calibration( char axis_char )
{
	unsigned char ee_addr;
	char          disp_buf[9];
	char		 *change_addr;
	long         *data_value_addr;
	long		  from_long;
	long		  to_long;
	char		  exception;
	long		  input_num;
	long		  long_value;
	char		  char_value;
	long		  data_value;

	/*------ Wrtite AXIS Calibration -----------------------------------------*/
	lcd_clear();
	led_off();
	lcd_locate( 0,0 );
	switch( axis_char ) {
		case 'X': ee_addr = EE_ADDR_X; break;
		case 'Y': ee_addr = EE_ADDR_Y; break;
		case 'Z': ee_addr = EE_ADDR_Z; break;
	}
	lcd_char( axis_char );
	lcd_string("-AXIS Calibration");
	lcd_locate( 0,1 );
	lcd_string( "Val=");
	long_value = ee_long_read(ee_addr+CAL_OFFSET);
	long_to_string( disp_buf, long_value, 5,0, 0x00 );
	lcd_string( disp_buf );
	Wait(1000);
	// 1st position
	do {
		switch( axis_char ) {
			case 'X': data_value = get_caliper_data_X(); break;
			case 'Y': data_value = get_caliper_data_Y(); break;
			case 'Z': data_value = get_caliper_data_Z(); break;
		}
		long_to_string( disp_buf, data_value, 3,2,0x00 );
		lcd_locate(0,2);
		lcd_string( disp_buf );
		from_long = data_value;
	} while( key_down( NO_WAIT ) != SET )
	key_up_anykey();

	// 2nd position
	do {
		switch( axis_char ) {
			case 'X': data_value = get_caliper_data_X(); break;
			case 'Y': data_value = get_caliper_data_Y(); break;
			case 'Z': data_value = get_caliper_data_Z(); break;
		}
		long_to_string( disp_buf, data_value, 3,2,0x00 );
		lcd_locate(10,2);
		lcd_string( disp_buf );
		to_long = data_value;
	} while( key_down( NO_WAIT ) != SET )
	key_up_anykey();		

	lcd_locate( 0,3 );
	input_num = get_numeric(&exception);

	if( exception != 'C' ) {
		long_value = input_num / ( input_num - abs(to_long - from_long) );
		lcd_clear_line( 3 );
		lcd_locate(0,3);
		lcd_string("Write...");
		long_to_string( disp_buf, long_value, 5,0, NO_SPAC );
		lcd_string( disp_buf );
		ee_long_write( ee_addr+CAL_OFFSET, long_value );
	} else {
		lcd_string("Cansel...");
	}
	Wait(2000);

	/*------ Increment Direct ------------------------------------------------*/
	lcd_clear();
	lcd_char( axis_char );	
	lcd_string("-Increment Direct");
	lcd_locate(0,1);
	lcd_string("Val=");
	char_value = ee_char_read( ee_addr + DIR_OFFSET );
	if( char_value == RIGHT ) {
		lcd_string( "RIGHT" );
	} else {
		lcd_string( "LEFT" );
	}
	lcd_locate(0,2);
	lcd_string("0:RIGHT 1:LEFT ");
	lcd_locate(0,3);
	exception = TRUE;
	do {
		switch( char_value = key_char(NO_SKIP) ) {
			case KEY_0:
				lcd_string("Write... RIGHT");				
				ee_char_write( ee_addr + DIR_OFFSET, RIGHT );
				exception = FALSE;
				break;   	
			case KEY_1:
				lcd_string("Write... LEFT");
				ee_char_write( ee_addr + DIR_OFFSET, LEFT  );
				exception = FALSE;
				break;
			case CANSEL:
				lcd_string("Cansel...");
				exception = FALSE;
				break;
		}
	} while( exception == TRUE )
	Wait(2000);

	/*------ magnification ---------------------------------------------------*/
	lcd_clear();
	lcd_char( axis_char );	
	lcd_string("-value is  x 2");
	lcd_locate(0,1);
	lcd_string("Val=");
	char_value = ee_char_read( ee_addr + MAG_OFFSET );
	if( char_value == MAG_X2 ) {
		lcd_string( "Val x 2.0" );
	} else {
		lcd_string( "Val x 1.0" );
	}
	lcd_locate(0,2);
	lcd_string("1:x1.0  2:x2.0");
	lcd_locate(0,3);
	exception = TRUE;
	do {
		switch( char_value = key_char(NO_SKIP) ) {
			case KEY_1:
				lcd_string("Write... x 1.0");				
				ee_char_write( ee_addr + MAG_OFFSET, MAG_X1 );
				exception = FALSE;
				break;   	
			case KEY_2:
				lcd_string("Write... x 2.0");
				ee_char_write( ee_addr + MAG_OFFSET, MAG_X2 );
				exception = FALSE;
				break;
			case CANSEL:
				lcd_string("Cansel...");
				exception = FALSE;
				break;
		}
	} while( exception == TRUE )
	Wait(2000);

	lcd_clear();
}


/*----------------------------------------------------------------------------*/
/* function: read from EEPROM char                                            */
/*                                                                            */
/*  arg1: EEPROM Address                                                      */
/*----------------------------------------------------------------------------*/
char ee_char_read( unsigned char addr )
{
	return ReadEEData( addr );
}


/*----------------------------------------------------------------------------*/
/* function: Write to from EEPROM char                                        */
/*                                                                            */
/*  arg1: EEPROM Address                                                      */
/*  arg2: char value                                                          */
/*----------------------------------------------------------------------------*/
void ee_char_write( unsigned char addr, char val )
{
	WriteEEData( addr, val );
}


/*----------------------------------------------------------------------------*/
/* function: read from EEPROM long                                            */
/*                                                                            */
/*  arg1: EEPROM Address                                                      */
/*----------------------------------------------------------------------------*/
long ee_long_read( unsigned char addr )
{
	LONG_CHAR	eedat;
	int			index;	

	for( index = 0; index < 4; index++ ) {
		eedat.c_dat[index] = ReadEEData( addr + index );
	}

	return eedat.l_dat;
}


/*----------------------------------------------------------------------------*/
/* function: Write to from EEPROM long                                        */
/*                                                                            */
/*  arg1: EEPROM Address                                                      */
/*  arg2: long value                                                          */
/*----------------------------------------------------------------------------*/
void ee_long_write( unsigned char addr, long long_val )
{
	LONG_CHAR	lc;
	int			index;
		
	lc.l_dat = long_val;
	for( index = 0; index < 4; index++ ) {
		WriteEEData( addr + index, lc.c_dat[index] );
	}
}


/*----------------------------------------------------------------------------*/
/* function: 7 SEG LED Output                                                 */
/*                                                                            */
/*  arg1: "-999.99" char *                                                    */
/*----------------------------------------------------------------------------*/
void led_out( unsigned char *led_str )
{
	unsigned int  led_bit_pat[3];
	unsigned char digit_cnt;
	unsigned char device_cnt;
	unsigned char bit_cnt;
	unsigned int  led_temp;

	led_bit_pat[0] = 0x0;
	led_bit_pat[1] = 0x0;	
	led_bit_pat[2] = 0x0;

	device_cnt = 0;
	for( digit_cnt = 0; digit_cnt < 6; digit_cnt++ ) {
		if( digit_cnt == 4 )
			led_str++;
		if( (digit_cnt % 2) == 0 ) {
			switch( *led_str ) {
				case '-': led_bit_pat[ device_cnt ] = LED_Dig1_MINUS; break;
				case '0': led_bit_pat[ device_cnt ] = LED_Dig1_0;     break;
				case '1': led_bit_pat[ device_cnt ] = LED_Dig1_1;     break;
				case '2': led_bit_pat[ device_cnt ] = LED_Dig1_2;     break;
				case '3': led_bit_pat[ device_cnt ] = LED_Dig1_3;     break;
				case '4': led_bit_pat[ device_cnt ] = LED_Dig1_4;     break;
				case '5': led_bit_pat[ device_cnt ] = LED_Dig1_5;     break;
				case '6': led_bit_pat[ device_cnt ] = LED_Dig1_6;     break;
				case '7': led_bit_pat[ device_cnt ] = LED_Dig1_7;     break;
				case '8': led_bit_pat[ device_cnt ] = LED_Dig1_8;     break;
				case '9': led_bit_pat[ device_cnt ] = LED_Dig1_9;     break;
			}
		} else {
			switch( *led_str ) {
				case '-': led_bit_pat[ device_cnt ] |= LED_Dig2_MINUS; break;
				case '0': led_bit_pat[ device_cnt ] |= LED_Dig2_0;     break;
				case '1': led_bit_pat[ device_cnt ] |= LED_Dig2_1;     break;
				case '2': led_bit_pat[ device_cnt ] |= LED_Dig2_2;     break;
				case '3': led_bit_pat[ device_cnt ] |= LED_Dig2_3;     break;
				case '4': led_bit_pat[ device_cnt ] |= LED_Dig2_4;     break;
				case '5': led_bit_pat[ device_cnt ] |= LED_Dig2_5;     break;
				case '6': led_bit_pat[ device_cnt ] |= LED_Dig2_6;     break;
				case '7': led_bit_pat[ device_cnt ] |= LED_Dig2_7;     break;
				case '8': led_bit_pat[ device_cnt ] |= LED_Dig2_8;     break;
				case '9': led_bit_pat[ device_cnt ] |= LED_Dig2_9;     break;
			}
			device_cnt++;
		}
		led_str++;
	}			

	led_bit_pat[1] |= LED_Dig2_DOT;

	switch( XYZ_counter ) {
		case 'X': led_bit_pat[0] |= LED_ON; break;
    	case 'Y': led_bit_pat[1] |= LED_ON; break;
		case 'Z': led_bit_pat[2] |= LED_ON; break;
	}
	
	SR_RCK  = LOW;
	SR_SCLR = LOW;
	SR_SCLR = HIGH;
	SR_G    = LOW;

	for( device_cnt = 0; device_cnt < 3; device_cnt++ ) {
		led_temp = led_bit_pat[device_cnt];
		for( bit_cnt = 0; bit_cnt < 16; bit_cnt++ ) {
			if( (led_temp & 0x0001) == 0x0001 ) {
				SR_SI = LOW;
			} else {
				SR_SI = HIGH;
			}
			SR_SCK = HIGH;
			SR_SCK = LOW;
			led_temp >>= 1;
		}
	}
	
	while( bit_is_clear( INTCON, TMR0IF ) ) {
		;
	}
	SR_RCK = HIGH;
}

/*----------------------------------------------------------------------------*/
/* function: 7 SEG LED OFF                                                    */
/*----------------------------------------------------------------------------*/
void led_off()
{
	SR_G    = HIGH;
}


/*----------------------------------------------------------------------------*/
/* function: Convert string -> long                                           */
/*                                                                            */
/*  arg1: string                                                              */
/*    format "-999.99\0" or "999.99\0"                                        */
/*            "-99.99\0" or  "99.99\0"                                        */
/*             "-9.99\0" or   "9.99\0"                                        */
/*----------------------------------------------------------------------------*/ 
long str_to_long( unsigned char *stp )
{
	long ret_val;
	long minus_flag;
	
	minus_flag = 1L;
	ret_val    = 0L;
	while( *stp != '\0' ) {
		switch( *stp ) {
			case '-': minus_flag = -1L; break
			case '0': ret_val *= 10L; ret_val += 0L; break;
			case '1': ret_val *= 10L; ret_val += 1L; break;
			case '2': ret_val *= 10L; ret_val += 2L; break;
			case '3': ret_val *= 10L; ret_val += 3L; break;
			case '4': ret_val *= 10L; ret_val += 4L; break;
			case '5': ret_val *= 10L; ret_val += 5L; break;
			case '6': ret_val *= 10L; ret_val += 6L; break;
			case '7': ret_val *= 10L; ret_val += 7L; break;
			case '8': ret_val *= 10L; ret_val += 8L; break;
			case '9': ret_val *= 10L; ret_val += 9L; break;
			case '.': break;
		}
		stp++;
	}
	
	return ret_val * minus_flag;
}	


/*----------------------------------------------------------------------------*/
/* function: long -> string                                                   */
/*                                                                            */
/*  arg1:pointer to string                                                    */
/*      format  "-000.00" or " 000.00"                                        */
/*           ex) -010.00                                                      */
/*               - 10.00   when fill zero == FLASE                            */
/*  arg2:long variable                                                        */
/*  arg3:fill zero                                                            */
/*----------------------------------------------------------------------------*/
void long_to_string(
	unsigned char  *stp,
	long          lval,
	unsigned char dp_up,
	unsigned char dp_below,
	unsigned char format )
{

	char index;
	char index2;
	char first_loop;
		
	if( dp_below > 0 ) {
		index = dp_up + dp_below + 1;
		*(stp + dp_up + 1 ) = '.';
	} else {
		index = dp_up;
	}
	*(stp + index + 1) = '\0';

	if( lval < 0L ) {
		*stp = '-';
		lval = -lval;
	} else {
		*stp = ' ';
	}	

	first_loop = TRUE;
	while( 1 <= index ) {
		if( lval == 0L && *(stp+index+1) != '\0') {
			if( bit_is_set(format,NO_SPAC) ) {
				index2 = 1;
				while( *(stp+index) != '\0' ) {
					*(stp+index2) = *(stp+index);
					index2++;
					index++;
				}
				*(stp+index2) = '\0'; 
				break;
			} else {
				if( index != (dp_up+1) ) {
					if( first_loop == TRUE ) {
						*(stp + index) = '0';
						first_loop = FALSE;
					} else {
						*(stp + index) = ' ';
					}
				}
				index--;
			}
		} else {
			*(stp + index) = '0' + (char)(lval % 10L);	
			index--;
			if( index == (dp_up+1) ) {
				index--;
			}
			lval = lval / 10L;
			first_loop = FALSE;
		}
	}	
}


/*----------------------------------------------------------------------------*/
/* function:Get long from keyboard                                            */
/*                                                                            */
/* arg1: Exception Code                                                       */
/*    'C':Cansel key enterd                                                   */
/*    'X':AXIS_X key enterd                                                   */
/*    'Y':AXIS_Y key enterd                                                   */
/*    'Z':AXIS_Z key enterd                                                   */
/*    'N':Not Exception                                                       */
/*----------------------------------------------------------------------------*/
long get_long( char *exception )
{
	unsigned char num_string[8];
	unsigned char key_code;
	unsigned char key_ch;
	unsigned char num_index;
	int	          num_counter;
	long 		  ret_val;

	num_counter = 3;	
	num_index   = 0;
	while( 1 ) {
		key_code = key_char( NO_SKIP );
		switch( key_code ) { 
			case KEY_1:     key_ch = '1'; break;
			case KEY_2:     key_ch = '2'; break;
			case KEY_3:     key_ch = '3'; break;
			case KEY_4:     key_ch = '4'; break;
			case KEY_5:     key_ch = '5'; break;
			case KEY_6:     key_ch = '6'; break;
			case KEY_7:     key_ch = '7'; break;
			case KEY_8:     key_ch = '8'; break;
			case KEY_9:     key_ch = '9'; break;
			case KEY_0:     key_ch = '0'; break;
			case KEY_MINUS: key_ch = '-'; break;
		}
		/*--- INPUT CANSEL ------------------------*/
		if( key_code == CANSEL ) {
			*exception = 'C';
			ret_val = 0L;
			break;
		}
		/*--- INPUT SET or HALF -------------------*/
		if( key_code == SET ) {
			num_string[num_index] = '\0';	
			ret_val = str_to_long( num_string );
			if( key_code == HALF ) {
				ret_val /= 2;
			}
			*exception = 'N';
			break;
		}
		/*--- INPUT (-) ---------------------------*/
		if( key_ch == '-' ) {
			if( num_index == 0 ) {
				lcd_char( key_ch );
				num_string[num_index] = key_ch;
				num_index++;	
			}
		}
		/*--- INPUT 0 to 9 -----------------------*/
		if( '0' <= key_ch && key_ch <= '9' ) {
			if( num_counter > 0 ) {
				lcd_char( key_ch );
				num_string[num_index] = key_ch;
				num_index++;
				num_counter--;
			}
		}
	}

	return ret_val;
}


/*----------------------------------------------------------------------------*/
/* function:Get numeric from keyboard                                         */
/*                                                                            */
/* arg1: Exception Code                                                       */
/*    'C':Cansel key enterd                                                   */
/*    'X':AXIS_X key enterd                                                   */
/*    'Y':AXIS_Y key enterd                                                   */
/*    'Z':AXIS_Z key enterd                                                   */
/*    'N':Not Exception                                                       */
/*----------------------------------------------------------------------------*/
long get_numeric( char *exception )
{
	unsigned char num_string[8];
	unsigned char key_code;
	unsigned char key_ch;
	unsigned char num_index;
	char		  dot_already;
	int	          num_counter;
	long 		  ret_val;

	num_counter = 3;	
	num_index   = 0;
	dot_already = FALSE;
	while( 1 ) {
		key_code = key_char( NO_SKIP );
		switch( key_code ) { 
			case KEY_1:     key_ch = '1'; break;
			case KEY_2:     key_ch = '2'; break;
			case KEY_3:     key_ch = '3'; break;
			case KEY_4:     key_ch = '4'; break;
			case KEY_5:     key_ch = '5'; break;
			case KEY_6:     key_ch = '6'; break;
			case KEY_7:     key_ch = '7'; break;
			case KEY_8:     key_ch = '8'; break;
			case KEY_9:     key_ch = '9'; break;
			case KEY_0:     key_ch = '0'; break;
			case KEY_DOT:   key_ch = '.'; break;
			case KEY_MINUS: key_ch = '-'; break;
		}
		/*--- INPUT CANSEL ------------------------*/
		if( key_code == CANSEL ) {
			*exception = 'C';
			ret_val = 99999;
			break;
		}
		/*--- INPUT AXIS_X ------------------------*/
		if( *exception == 'X' && key_code == AXIS_X ) {
			*exception = 'X';
			ret_val = 99999;
			break;
		}
		/*--- INPUT AXIS_Y ------------------------*/
		if( *exception == 'Y' && key_code == AXIS_Y ) {
			*exception = 'Y';
			ret_val = 99999;
			break;
		}
		/*--- INPUT AXIS_Z ------------------------*/
		if( *exception == 'Z' && key_code == AXIS_Z ) {
			*exception = 'Z';
			ret_val = 99999;
			break;
		}		
		/*--- INPUT SET or HALF -------------------*/
		if( key_code == SET || key_code == HALF ) {
			if( dot_already == FALSE ) {
				num_string[num_index++] = '.';
				num_string[num_index++] = '0';
				num_string[num_index++] = '0';
			} else {
				while( num_counter > 0 ) {
					num_string[num_index++] = '0';
					num_counter--;
				}
			}
			num_string[num_index] = '\0';	
			ret_val = str_to_long( num_string );
			if( key_code == HALF ) {
				ret_val /= 2;
			}
			*exception = 'N';
			break;
		}
		/*--- INPUT HALF --------------------------*/
		if( key_code == SET ) {
			num_string[num_index] = '\0';
			
			lcd_string( num_string );			
			break;
		}				
		/*--- INPUT (-) ---------------------------*/
		if( key_ch == '-' ) {
			if( num_index == 0 ) {
				lcd_char( key_ch );
				num_string[num_index] = key_ch;
				num_index++;	
			}
		}
		/*--- INPUT 0 to 9 -----------------------*/
		if( '0' <= key_ch && key_ch <= '9' ) {
			if( num_counter > 0 ) {
				lcd_char( key_ch );
				num_string[num_index] = key_ch;
				num_index++;
				num_counter--;
			}
		}
		/*--- INPUT (.) ---------------------------*/
		if( key_ch == '.' ) {
			if( dot_already == FALSE ) {
				lcd_char( key_ch );
				num_string[num_index] = key_ch;
				num_index++;
				num_counter = 2;
				dot_already = TRUE;
			}
		}											
	}

	return ret_val;
}

/*----------------------------------------------------------------------------*/
/* function:Get char from keyboard                                            */
/*                                                                            */
/*  arg1:                                                                     */
/*    SKIP -> skip key up                                                     */
/*----------------------------------------------------------------------------*/
unsigned char key_char( unsigned char skip )
{
	if( !(skip == SKIP) )
		key_up_anykey();
	return key_down( WAIT );
}	

void key_up_anykey()
{
	unsigned char key_stat; 

	Wait(50);
	KEY_Col1 = LOW;
	KEY_Col2 = LOW;
	KEY_Col3 = LOW;
	KEY_Col4 = LOW;
	while( KEY_Row1 == LOW ||
	       KEY_Row2 == LOW ||
	       KEY_Row3 == LOW ||
	       KEY_Row4 == LOW ||
	       KEY_Row5 == LOW ) {
		;
	}
	Wait(20);
}

/*
void key_up()
{
	unsigned char key_stat; 

	KEY_Col1 = HIGH;
	KEY_Col2 = HIGH;
	KEY_Col3 = HIGH;
	KEY_Col4 = HIGH;
	if( Key_col == 1 ) { KEY_Col1 = LOW; }
	if( Key_col == 2 ) { KEY_Col2 = LOW; }
	if( Key_col == 3 ) { KEY_Col3 = LOW; }
	if( Key_col == 4 ) { KEY_Col4 = LOW; }
	do {
		if( Key_row == 1 ) { key_stat = KEY_Row1; }
		if( Key_row == 2 ) { key_stat = KEY_Row2; }
		if( Key_row == 3 ) { key_stat = KEY_Row3; }
		if( Key_row == 4 ) { key_stat = KEY_Row4; }
		if( Key_row == 5 ) { key_stat = KEY_Row5; }
	} while( key_stat == LOW );
	Wait(10);
}
*/

/*----------------------------------------------------------------------------*/
/* function: key down detect                                                  */
/*                                                                            */
/*  arg1:                                                                     */
/*     WAIT    -> Wait key down                                               */
/*     NO_WAIT -> No Wait key down                                            */
/*----------------------------------------------------------------------------*/ 
unsigned char key_down( unsigned char wait_flag )
{
	Key_col = 1;
	while( 1 ) {
		KEY_Col1 = HIGH;
		KEY_Col2 = HIGH;
		KEY_Col3 = HIGH;
		KEY_Col4 = HIGH;	
		if( Key_col == 1 ) { KEY_Col1 = LOW; }
		if( Key_col == 2 ) { KEY_Col2 = LOW; }
		if( Key_col == 3 ) { KEY_Col3 = LOW; }
		if( Key_col == 4 ) { KEY_Col4 = LOW; }
		Key_row = 0;
		if( KEY_Row1 == LOW ) { Key_row = 1; }
		if( KEY_Row2 == LOW ) { Key_row = 2; }
		if( KEY_Row3 == LOW ) { Key_row = 3; }
		if( KEY_Row4 == LOW ) { Key_row = 4; }
		if( KEY_Row5 == LOW ) { Key_row = 5; }
		if( Key_row != 0 )
			break;
		Key_col++;	
		if( Key_col == 5 ) {
			if( wait_flag == WAIT ) {
				Key_col = 1;
			} else {
				return KEY_NON;
			}
		}
	}
	beep(5);
//	if( wait_flag == WAIT ) {
		Wait(10);
//	}
	
	return (Key_row-1)*4 + (Key_col-1) + 1;
}


/*----------------------------------------------------------------------------*/
/* Function: wait micro sec                                                   */
/*                                                                            */
/* arg1: wait us time                                                         */
/*----------------------------------------------------------------------------*/


#pragma asm
#pragma asmdefine _GetSPOtoFSR0

	FreqNS		equ		(D'1000000000'/_APROCFREQ)*4
	LowCyc		equ		D'1000'/FreqNS

NOP_US macro delayus
	loopcnt=delayus
	while loopcnt-1>0
		NOP
		loopcnt-=1
	endw
endm

module "wait_us"
wait_us::
;		SETPCLATH 0,wait_us,1
		if _QUICKCALL==1
			MGETFSRSPO 1		; Set FSR to point to 1st parameter
		else
			MGETFSRSPO 3		; (When software stack is used)
		endif
		movf POSTINC0,w
		movwf ACC		

;		SETPCLATH wait_us,0,1		
wait_us_loop:
		NOP_US LowCyc
		decfsz ACC
		goto wait_us_loop

		MRET 0
endmodule
#pragma asmend	


/*----------------------------------------------------------------------------*/
/* Function:Get caliper Data from X-AXIS                                      */
/*                                                                            */
/* Return:(long)X-AXIS ReadOut Data                                           */
/*----------------------------------------------------------------------------*/
long get_caliper_data_X()
{
	unsigned char d1,d2,d3,d4;
	char   		count;
	LONG_CHAR	temp;

	d1 = 0x00;
	d2 = 0x00;
	d3 = 0x00;
	d4 = 0x00;

start:
	while( X_DATA == LOW  ) { ;	}
	while( X_DATA == HIGH ) { ;	}
	if( X_CLOCK == HIGH ) {
	 	while( X_CLOCK == HIGH ) { ; }
		// get 1st byte
		for( count  = 0 ; count < 8 ; count++ ) {
			d1 >>= 1;
			while( X_CLOCK == LOW ) { ; }
			if( X_DATA == HIGH ) {
				d1 |= 0x80;
			}
			while( X_CLOCK == HIGH ) { ; }
		}
		// get 2nd byte
		for( count  = 0 ; count < 8 ; count++ ) {
			d2 >>= 1;
			while( X_CLOCK == LOW ) { ; }
			if( X_DATA == HIGH ) {
				d2 |= 0x80;
			}
			while( X_CLOCK == HIGH ) { ; }
		}
		// get 3rd byte
		for( count  = 0 ; count < 8 ; count++ ) {
			d3 >>= 1;
			while( X_CLOCK == LOW ) { ; }
			if( X_DATA == HIGH ) {
				d3 |= 0x80;
			}

			while( X_CLOCK == HIGH ) { ; }
		}
		// get 4th byte
		for( count  = 0 ; count < 8 ; count++ ) {
			d4 >>= 1;
			while( X_CLOCK == LOW ) { ; }
			if( X_DATA == HIGH ) {
				d4 |= 0x80;
			}
			while( X_CLOCK == HIGH ) { ; }
		}
		// error
		while( X_CLOCK == LOW ) { ; }
		while( X_DATA  == LOW ) { ; }
		if( X_CLOCK != HIGH ) {
			return 88888;
		}
	} else {
		goto start;
	}

	temp.c_dat[0] = d1;
	temp.c_dat[1] = d2;
	temp.c_dat[2] = d3;
	temp.c_dat[3] = d4;

	return temp.l_dat;	
}


/*----------------------------------------------------------------------------*/
/* Function:Get caliper Data from Y-AXIS                                      */
/*                                                                            */
/* Return:(long)Y-AXIS ReadOut Data                                           */
/*----------------------------------------------------------------------------*/
long get_caliper_data_Y()
{
	unsigned char d1,d2,d3,d4;
	char   		count;
	LONG_CHAR	temp;

	d1 = 0x00;
	d2 = 0x00;
	d3 = 0x00;
	d4 = 0x00;

start:
	while( Y_DATA == LOW  ) { ;	}
	while( Y_DATA == HIGH ) { ;	}
	if( Y_CLOCK == HIGH ) {
	 	while( Y_CLOCK == HIGH ) { ; }
		// get 1st byte
		for( count  = 0 ; count < 8 ; count++ ) {
			d1 >>= 1;
			while( Y_CLOCK == LOW ) { ; }
			if( Y_DATA == HIGH ) {
				d1 |= 0x80;
			}
			while( Y_CLOCK == HIGH ) { ; }
		}
		// get 2nd byte
		for( count  = 0 ; count < 8 ; count++ ) {
			d2 >>= 1;
			while( Y_CLOCK == LOW ) { ; }
			if( Y_DATA == HIGH ) {
				d2 |= 0x80;
			}
			while( Y_CLOCK == HIGH ) { ; }
		}
		// get 3rd byte
		for( count  = 0 ; count < 8 ; count++ ) {
			d3 >>= 1;
			while( Y_CLOCK == LOW ) { ; }
			if( Y_DATA == HIGH ) {
				d3 |= 0x80;
			}

			while( Y_CLOCK == HIGH ) { ; }
		}
		// get 4th byte
		for( count  = 0 ; count < 8 ; count++ ) {
			d4 >>= 1;
			while( Y_CLOCK == LOW ) { ; }
			if( Y_DATA == HIGH ) {
				d4 |= 0x80;
			}
			while( Y_CLOCK == HIGH ) { ; }
		}
		// error
		while( Y_CLOCK == LOW ) { ; }
		while( Y_DATA  == LOW ) { ; }
		if( Y_CLOCK != HIGH ) {
			return 88888;
		}	
	} else {
		goto start;
	}

	temp.c_dat[0] = d1;
	temp.c_dat[1] = d2;
	temp.c_dat[2] = d3;
	temp.c_dat[3] = d4;

	return temp.l_dat;	
}


/*----------------------------------------------------------------------------*/
/* Function:Get caliper Data from Z-AXIS                                      */
/*                                                                            */
/* Return:(long)Z-AXIS ReadOut Data                                           */
/*----------------------------------------------------------------------------*/
long get_caliper_data_Z()
{
	unsigned char d1,d2,d3,d4;
	char   		count;
	LONG_CHAR	temp;

	d1 = 0x00;
	d2 = 0x00;
	d3 = 0x00;
	d4 = 0x00;

start:
	while( Z_DATA == LOW  ) { ;	}
	while( Z_DATA == HIGH ) { ;	}
	if( Z_CLOCK == HIGH ) {
	 	while( Z_CLOCK == HIGH ) { ; }
		// get 1st byte
		for( count  = 0 ; count < 8 ; count++ ) {
			d1 >>= 1;
			while( Z_CLOCK == LOW ) { ; }
			if( Z_DATA == HIGH ) {
				d1 |= 0x80;
			}
			while( Z_CLOCK == HIGH ) { ; }
		}
		// get 2nd byte
		for( count  = 0 ; count < 8 ; count++ ) {
			d2 >>= 1;
			while( Z_CLOCK == LOW ) { ; }
			if( Z_DATA == HIGH ) {
				d2 |= 0x80;
			}
			while( Z_CLOCK == HIGH ) { ; }
		}
		// get 3rd byte
		for( count  = 0 ; count < 8 ; count++ ) {
			d3 >>= 1;
			while( Z_CLOCK == LOW ) { ; }
			if( Z_DATA == HIGH ) {
				d3 |= 0x80;
			}

			while( Z_CLOCK == HIGH ) { ; }
		}
		// get 4th byte
		for( count  = 0 ; count < 8 ; count++ ) {
			d4 >>= 1;
			while( Z_CLOCK == LOW ) { ; }
			if( Z_DATA == HIGH ) {
				d4 |= 0x80;
			}
			while( Z_CLOCK == HIGH ) { ; }
		}
		// error
		while( Z_CLOCK == LOW ) { ; }
		while( Z_DATA  == LOW ) { ; }
		if( Z_CLOCK != HIGH ) {
			return 88888;
		}	
	} else {
		goto start;
	}

	temp.c_dat[0] = d1;
	temp.c_dat[1] = d2;
	temp.c_dat[2] = d3;
	temp.c_dat[3] = d4;

	return temp.l_dat;	
}


/*----------------------------------------------------------------------------*/
/* function: LCD clear Line                                                   */
/*                                                                            */
/*  arg1: 0-3                                                                 */ 
/*----------------------------------------------------------------------------*/
void lcd_clear_line( unsigned char y )
{
	unsigned char x;

	lcd_locate( 0, y );
	for( x = 0 ; x <= 19 ; x++ )
		lcd_char(' ');
}

/*----------------------------------------------------------------------------*/
/*  function: send to LCD write signal                                        */
/*----------------------------------------------------------------------------*/
void lcd_enable()
{
	LCD_E   = 1;
	wait_us(20);
	LCD_E   = 0;
	wait_us(20);
}


/*----------------------------------------------------------------------------*/
/* function:display string to LCD                                             */
/*                                                                            */
/*  arg1:display string                                                       */
/*----------------------------------------------------------------------------*/
void lcd_string(unsigned char *str)
{
	while( *str != '\0' ) {
		lcd_char( *str );
		str++;
	}
}


/*----------------------------------------------------------------------------*/
/*  function: send 1 char to LCD                                              */
/*----------------------------------------------------------------------------*/
void lcd_char(unsigned char ch)
{
	LCD_RS = 1;
	LCD_DB7 = (ch & (1<<7)) != 0 ? 1 : 0;
	LCD_DB6 = (ch & (1<<6)) != 0 ? 1 : 0;
	LCD_DB5 = (ch & (1<<5)) != 0 ? 1 : 0;
	LCD_DB4 = (ch & (1<<4)) != 0 ? 1 : 0;
	lcd_enable();
	LCD_DB7 = (ch & (1<<3)) != 0 ? 1 : 0;
	LCD_DB6 = (ch & (1<<2)) != 0 ? 1 : 0;
	LCD_DB5 = (ch & (1<<1)) != 0 ? 1 : 0;
	LCD_DB4 = (ch & (1<<0)) != 0 ? 1 : 0;
	lcd_enable();
	LCD_RS = 0;
}

/*----------------------------------------------------------------------------*/
/*  function:clear LCD                                                        */
/*----------------------------------------------------------------------------*/
void lcd_clear()
{
	LCD_DB7 = 0; LCD_DB6 = 0; LCD_DB5 = 0; LCD_DB4 = 0;
	lcd_enable();
	LCD_DB7 = 0; LCD_DB6 = 0; LCD_DB5 = 0; LCD_DB4 = 1;
	lcd_enable();
	Wait(2);
}

/*----------------------------------------------------------------------------*/
/*  function: locate LCD cursole (20*4)                                       */
/*                                                                            */
/*  arg1:X 0-19                                                               */
/*  arg2:Y 0-3                                                                */
/*----------------------------------------------------------------------------*/
void lcd_locate( unsigned char x, unsigned char y )
{
	unsigned char dat;
	
	LCD_RS = 0;
	LCD_RW = 0;
	switch( y ) {
		case 0: dat = 0x80        + x; break;
		case 1: dat = 0x80 + 0x40 + x; break;
		case 2: dat = 0x80 + 0x14 + x; break;
		case 3: dat = 0x80 + 0x54 + x; break;
	}
	LCD_DB7 = (dat & (1<<7)) != 0 ? 1 : 0;
	LCD_DB6 = (dat & (1<<6)) != 0 ? 1 : 0;
	LCD_DB5 = (dat & (1<<5)) != 0 ? 1 : 0;
	LCD_DB4 = (dat & (1<<4)) != 0 ? 1 : 0;
	lcd_enable();
	LCD_DB7 = (dat & (1<<3)) != 0 ? 1 : 0;
	LCD_DB6 = (dat & (1<<2)) != 0 ? 1 : 0;
	LCD_DB5 = (dat & (1<<1)) != 0 ? 1 : 0;
	LCD_DB4 = (dat & (1<<0)) != 0 ? 1 : 0;
	lcd_enable();
	wait_us(60);
}

/*----------------------------------------------------------------------------*/
/*  function:LCD initialize                                                   */
/*----------------------------------------------------------------------------*/
void lcd_init()
{
	LCD_E = 0;
	Wait(500);
	LCD_RS = 0;
	LCD_RW = 0;
	LCD_DB7 = 0; LCD_DB6 = 0; LCD_DB5 = 1; LCD_DB4 = 1;
	lcd_enable();
	Wait(4);
	lcd_enable();
	lcd_enable();
	LCD_DB7 = 0; LCD_DB6 = 0; LCD_DB5 = 1; LCD_DB4 = 0;
	lcd_enable();
	lcd_enable();
	LCD_DB7 = 1; LCD_DB6 = 0; LCD_DB5 = 0; LCD_DB4 = 0;
	lcd_enable();

// Entry Mode Set
	LCD_DB7 = 0; LCD_DB6 = 0; LCD_DB5 = 0; LCD_DB4 = 0;
	lcd_enable();
	LCD_DB7 = 0; LCD_DB6 = 1; LCD_DB5 = 1; LCD_DB4 = 0;
	lcd_enable();

// Display ON
	LCD_DB7 = 0; LCD_DB6 = 0; LCD_DB5 = 0; LCD_DB4 = 0;
	lcd_enable();
	LCD_DB7 = 1; LCD_DB6 = 1; LCD_DB5 = 1; LCD_DB4 = 0;
	lcd_enable();

// Display CLS	
	lcd_clear();
}
